
#include <avr/io.h>
#include <inttypes.h>
#include <pgmspace.h>
#include <avr/sleep.h>

#include <spi.h>
#include <can.h>
#include <mcp2515.h>
#include <buskoppler.h>

#include <avr/interrupt.h>
#include <avr/signal.h>


#include <scheduler.h>

#include <taster.h>
#include <relais.h>
#include <dimmer.h>
#include <buskoppler.h>

#include <tab.h>
//#include <graphik.h>

#include <lcd.h>

#include <lcd_fnt3.h>


// #define RECONFIGURE_SPI


/************************************************************************/
/*
/*
/*
/************************************************************************/


/************************************************************************/
/*
/*
/*
/************************************************************************/

void wait_us(uint16_t us){
  do{
    asm("nop");
    asm("nop");
    asm("nop");
    asm("nop");
  } while(--us);
}


void display_gets_spi(void){
  SPCR      = 0x5D;		// CPOL = 1, CPHA = 1; 1Mhz (in double-Speed Master Mode)
}


void display_releases_spi(void){
  SPCR      = 0x50;		// CPOL = 0, CPHA = 0; 8Mhz (in double-Speed Master Mode)
}


/************************************************************************/
/*
/*
/*
/************************************************************************/

void display_xdata(uint8_t *data_ptr, uint16_t data_anz){
  while(data_anz--){
    #ifdef RECONFIGURE_SPI
    cli();			// IRs sperren wegen umkonfiguriertem SPI
    SPCR      = 0x5D;		// CPOL = 1, CPHA = 1; 1Mhz (in double-Speed Master Mode)
    #endif
    disp_rs_port |= _BV(disp_rs_bit);
    spi_port &= ~_BV(disp_cs_bit);
    SPDR = *data_ptr++;
    while(!(SPSR & _BV(SPIF)));
    spi_port |= _BV(disp_cs_bit);
//    wait_us(10);
    #ifdef RECONFIGURE_SPI
    SPCR      = 0x50;	// CPOL = 0, CPHA = 0; 8Mhz (in double-Speed Master Mode)
    sei();			// IRs wieder freigeben
    #endif
  }
}


void display_pgm_xdata(const uint8_t *data_ptr, uint16_t data_anz){
  while(data_anz--){
    #ifdef RECONFIGURE_SPI
    cli();			// IRs sperren wegen umkonfiguriertem SPI
    SPCR      = 0x5D;		// CPOL = 1, CPHA = 1; 1Mhz (in double-Speed Master Mode)
    #endif
    disp_rs_port |= _BV(disp_rs_bit);
    spi_port &= ~_BV(disp_cs_bit);
    SPDR = pgm_read_byte(data_ptr++);
    while(!(SPSR & _BV(SPIF)));
    spi_port |= _BV(disp_cs_bit);
//    wait_us(10);
    #ifdef RECONFIGURE_SPI
    SPCR      = 0x50;	// CPOL = 0, CPHA = 0; 8Mhz (in double-Speed Master Mode)
    sei();			// IRs wieder freigeben
    #endif
  }
}


void display_cmd(uint8_t cmd){
  #ifdef RECONFIGURE_SPI
  cli();			// IRs sperren wegen umkonfiguriertem SPI
  SPCR      = 0x5D;		// CPOL = 1, CPHA = 1; 1Mhz (in double-Speed Master Mode)
  #endif
    disp_rs_port &= ~_BV(disp_rs_bit);
  spi_port &= ~_BV(disp_cs_bit);
  SPDR = cmd;
  while(!(SPSR & _BV(SPIF)));
  spi_port |= _BV(disp_cs_bit);
//  wait_us(10);
  #ifdef RECONFIGURE_SPI
  SPCR      = 0x50;		// CPOL = 0, CPHA = 0; 8Mhz (in double-Speed Master Mode)
  sei();			// IRs wieder freigeben
  #endif
}


/************************************************************************/
/*
/*
/*
/************************************************************************/

void display_set_xy(uint8_t x, uint8_t y){
//  wait_us(100);
  display_cmd(0x00 + (x & 0x0F));
//  wait_us(100);
  display_cmd(0x10 + (x >>   4));
//  wait_us(100);
  display_cmd(0xB0 + (y & 0x07));
//  wait_us(100);

}


#define DISPLAY_X_SIZE	132	// Pixelbreite des Displays
#define DISPLAY_Y_BSIZE	8	// Byte-Hhe des Displays


void display_clear(void){
  uint8_t x,y;
  uint8_t my_zero = 0;
  for (y=0; y<DISPLAY_Y_BSIZE; y++){
    display_set_xy(0, y);
    for (x=0; x<DISPLAY_X_SIZE; x++){
      display_set_xy(x, y);
      display_xdata(&my_zero, 1);
    }
  }
}


PROGMEM const uint8_t gdata[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

void display_test(void){
  display_set_xy(30, 3);
  wait_us(50);
  display_pgm_xdata(gdata, 8);
  display_set_xy(50, 3);
  wait_us(50);
  display_pgm_xdata(gdata, 8);
  display_set_xy(70, 3);
  wait_us(50);
  display_pgm_xdata(gdata, 8);
}


/************************************************************************/
/*
/*
/*
/************************************************************************/

void display_init(void){

  SPCR      = 0x5D;		// CPOL = 1, CPHA = 1; 1Mhz (in double-Speed Master Mode)

  disp_rs_ddr     |=  _BV(disp_rs_bit);
  disp_led_ddr    |=  _BV(disp_led_bit);
  disp_led_port   &= ~_BV(disp_led_bit);

  disp_reset_ddr  |=  _BV(disp_reset_bit);
  disp_reset_port &= ~_BV(disp_reset_bit);
  wait_us(10000);
  disp_reset_port |=  _BV(disp_reset_bit);
  wait_us(10000);
  display_cmd(0xA0);
  wait_us(10000);
  display_cmd(0xC8);
  wait_us(10000);
  display_cmd(0xA2);	// LCD-Bias ???
  wait_us(10000);

  display_cmd(0x2F);	// Power control VC = on, VR = on, VF = on
  wait_us(10000);
  display_cmd(0x25);	// Regulator resistor select  ???
  wait_us(10000);
  display_cmd(0x81);	// reference voltage mode
  display_cmd(0x1F);	// reference voltage ???
  wait_us(10000);
/*
  display_cmd(0xE2);	// Reset Display
  wait_us(10000);

  display_cmd(0x25);	// Regulator resistor select  ???
  wait_us(10000);
  display_cmd(0x81);	// reference voltage mode
  display_cmd(0x1F);	// reference voltage ???
  wait_us(10000);
*/
  display_cmd(0x40);	// Initial Display line = 0
  wait_us(10000);
  display_cmd(0xAF);	// Display on
  wait_us(10000);
//  display_cmd(0xA5);	// Entire Display on
//  wait_us(10000);

  display_clear();
//  display_test();
//  paint_graphik(40, 30, 0, &test_graphik);

  lcd_direktout = 1;

/*
  setline( 0,  0, 30,  1);
  setline( 0,  0,  1, 10);
  setline( 0, 10, 30, 11);
  setline(30,  0, 31, 11);

  setline(10, 20, 100, 23);
  invert (11, 21, 99, 22);
*/

//  or_graphik(8, 10, &lcd_fnt1_0x53);

  //set_char('A', 10, 10, lcd_fnt3);

//  set_text("7\x1BI\x33 8\x1Bi9", 0, 5, lcd_fnt3);
  set_text("7\x1BI\x20 8 9", 0, 5, lcd_fnt3);
  // set_text("78", 10, 5, lcd_fnt3);
  set_text("123", 10, 35, lcd_fnt3);

//  set_text("\x1BI\x33 TEST ", 10, 10, lcd_fnt3);

//  set_text("\x1BI\x33 TEST ", 10, 10, lcd_fnt3);
//  invert(10, 7, 30, 21);  //x, inv_oben, x + pgm_read_byte(&(g_ptr->x)), inv_unten);    //	((graphik_typ*)(*fptr)[(unsigned char)c])->x, y2);

//   lcd_graphik(0, 0, XSIZE-1, YSIZE-1);      	// auf Display ausgeben

  while(1);

}



// end of file


