
#include <avr/io.h>
#include <inttypes.h>
#include <pgmspace.h>
#include <avr/sleep.h>

#include <spi.h>
#include <can.h>
#include <mcp2515.h>
#include <display.h>

#include <scheduler.h>
#include <buskoppler.h>


/************************************************************************/
/*
/*
/*
/************************************************************************/

typedef struct{
  uint8_t x_links;	// X-Start des Paint-Fensters (in Pixel)
  uint8_t x_rechts;	// X-Ende (inklusive) des Paint-Fensters (in Pixel)
  uint8_t yb_oben;	// Y-Start des Paint-Fensters (in Bytes)
  uint8_t yb_unten;	// Y-Ende (inklusive) des Paint-Fensters (in Bytes)
}  paint_typ;

typedef struct{
  uint8_t x;		// Lnge der Garphik in Pixeln
  uint8_t y;		// Hhe der Graphik in Pixeln
  uint8_t data[];	// Graphik-Information
}  graphik_typ;

typedef graphik_typ *graphik_ptr;


#define paint_mem_size (4*64)

uint8_t paint_area[paint_mem_size];

paint_typ paint_bounds = {32, 95, 2, 5};

// uint8_t paint_x;
// uint8_t paint_y;

const graphik_typ test_graphik PROGMEM = {
  10,
  10,
  { 0xFF, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0xFF,
    0xC0, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0xC0}
};

#define max(A,B) ((A) > (B) ? (A) : (B))
#define min(A,B) ((A) < (B) ? (A) : (B))


void paint_graphik(uint8_t x_links, uint8_t y_oben, uint8_t mode, const graphik_ptr g){
  uint8_t yb_oben  = y_oben >> 3;
  uint8_t yb_unten = ((y_oben + pgm_read_byte(&(g->y)) + 7) >> 3);
  uint8_t x_rechts = x_links +  pgm_read_byte(&(g->x));

  if ( (yb_oben  > paint_bounds.yb_unten) ||
       (yb_unten < paint_bounds.yb_oben)  ||
       (x_links  > paint_bounds.x_rechts) ||
       (x_rechts < paint_bounds.x_links)  )
  {
    return;
  }

  uint8_t yb           = max(yb_oben,  paint_bounds.yb_oben);
  uint8_t yb_ende      = min(yb_unten, paint_bounds.yb_unten);
  uint8_t x_start      = max(x_links,  paint_bounds.x_links);
  uint8_t x_ende       = min(x_rechts, paint_bounds.x_rechts);
  uint8_t paint_breite = paint_bounds.x_rechts - paint_bounds.x_links + 1;
  uint8_t paint_ofs    = (yb - paint_bounds.yb_oben) * paint_breite;
  uint8_t graphik_ofs  = (yb - yb_oben) * pgm_read_byte(&(g->x));
  uint8_t x;
  uint8_t yl_shift;
  uint8_t yr_shift;
  uint8_t b;

  if(y_oben & 0x07){
    yl_shift = y_oben & 0x07;
    yr_shift = 8-yl_shift;

    while (yb < yb_ende){
      x = x_start;
      while (x < x_ende){
        b = pgm_read_byte(&(g->data[graphik_ofs +x]));
        paint_area[paint_ofs + x] |= (b >> yl_shift);
        if ((yb+1) < paint_bounds.yb_unten){
          paint_area[paint_ofs + paint_breite + x] |= (b << yr_shift);
        }
        x++;
      }
      paint_ofs   += paint_breite;
      graphik_ofs +=  pgm_read_byte(&(g->x));
      yb++;
    }
  }

  else{

    while (yb < yb_ende){
      x = x_start;
      while (x < x_ende){
        b = pgm_read_byte(&(g->data[graphik_ofs +x]));
        paint_area[paint_ofs + x] |= b;
        x++;
      }
      paint_ofs   += paint_breite;
      graphik_ofs +=  pgm_read_byte(&(g->x));
      yb++;
    }
  }
}



