
#include <avr/io.h>
#include <inttypes.h>
#include <pgmspace.h>
#include <avr/sleep.h>

#include <can.h>
#include <mcp2515.h>

#include <scheduler.h>


/************************************************************************/
/*
/* Relais.c
/*
/************************************************************************/

uint8_t rel1 = 0;
uint8_t rel2 = 0;
uint8_t rel3 = 0;

void clear_relais(uint8_t nr){
  if (nr < 8){
    rel1 &= ~_BV(7-nr);
  }
  else{
    if (nr < 16){
      rel2 &= ~_BV(15-nr);
    }
    else{
      rel3 &= ~_BV(23-nr);
    }
  }
}


void set_relais(uint8_t nr){
  uint8_t mybit = 1;
  if (nr < 8){
  rel1 |= (uint8_t)(mybit << (uint8_t)(7-nr));
  }
  else{
    if (nr < 16){
      rel2 |= _BV(15-nr);
    }
    else{
      rel3 |= _BV(23-nr);
    }
  }
}


/************************************************************************/
/*
/*
/*
/************************************************************************/

void init_relais(const module_parm_typ *my_module){
  module_var_typ *var_ptr = (module_var_typ*)pgm_read_word(&my_module->var);
  var_ptr->state = 0;
  clear_relais(pgm_read_byte(&(my_module->hw_nr)));
}


void timer_relais(const module_parm_typ *my_module){
  module_var_typ *var_ptr = (module_var_typ*)pgm_read_word(&my_module->var);
  if (var_ptr->state == 1){
    var_ptr->state = 0;
    clear_relais(pgm_read_byte(&(my_module->hw_nr)));
    // spi_relais(rel1, rel2, rel3);
  }
}


void msg_relais(const module_parm_typ *my_module, can_msg_typ *my_msg){
  module_var_typ *var_ptr = (module_var_typ*)pgm_read_word(&my_module->var);
  uint8_t my_hw_nr        = pgm_read_byte(&(my_module->hw_nr));

/*
  if (my_msg->data[0] == set_state){
    if ((var_ptr->state = my_msg->data[1]) > 0){
       set_relais(my_hw_nr);
    }
    else{
      clear_relais(my_hw_nr);
    }
    // spi_relais(rel1, rel2, rel3);
  }
*/
// if (my_msg->data[0] == get_state){}   // msg mit state-Zustand verschicken

  if (var_ptr->state > 0){
    if ( ((*my_msg)[can_msg_data] == key_down_pressed) ||
         ((*my_msg)[can_msg_data] == key_switch_pressed))
    {
      var_ptr->state = 0;
      clear_relais(my_hw_nr);
      // spi_relais(rel1, rel2, rel3);
    }
    else{
      if ((*my_msg)[can_msg_data] == key_pressed){
        var_ptr->state = 2;
      }
    }
  }
  else{
    if ( ((*my_msg)[can_msg_data] == key_pressed) ||
         ((*my_msg)[can_msg_data] == key_switch_pressed))
    {
      var_ptr->state = 1;
      set_relais(my_hw_nr);
      // spi_relais(rel1, rel2, rel3);
    }
  }
  if ( ( ((*my_msg)[can_msg_data] == key_long_released) ||
         ((*my_msg)[can_msg_data] == key_switch_long_released)
       ) && (var_ptr->state == 1) )
  {
    var_ptr->state = 0;
    clear_relais(my_hw_nr);
    // spi_relais(rel1, rel2, rel3);
  }
}


/************************************************************************/
/*
/* Jalousie-Funktionen:
/*
/* Parameter 1: Zeit
/*
/************************************************************************/

#define jalousie_all_up		1
#define jalousie_all_down	2
#define jalousie_manual_up	3
#define jalousie_manual_down	4

#define t_second		50


void init_jalousie(const module_parm_typ *my_module){
  module_var_typ *var_ptr = (module_var_typ*)pgm_read_word(&my_module->var);
  uint8_t my_hw_nr        = pgm_read_byte(&(my_module->hw_nr));
  var_ptr->state = 0;
  clear_relais(my_hw_nr);
  clear_relais(my_hw_nr+1);
}


void timer_jalousie(const module_parm_typ *my_module){
  module_var_typ *var_ptr = (module_var_typ*)pgm_read_word(&my_module->var);
  uint8_t my_hw_nr        = pgm_read_byte(&(my_module->hw_nr));
  if (var_ptr->sek){
    var_ptr->sek--;
    var_ptr->timer = t_second;
  }

  if (var_ptr->sek == 0){
    var_ptr->timer = 0;
    var_ptr->state = 0;
    clear_relais(my_hw_nr);
    clear_relais(my_hw_nr+1);
    // spi_relais(rel1, rel2, rel3);
  }
}


void msg_jalousie(const module_parm_typ *my_module, can_msg_typ *my_msg){
  module_var_typ *var_ptr = (module_var_typ*)pgm_read_word(&my_module->var);
  uint8_t my_hw_nr        = pgm_read_byte(&(my_module->hw_nr));

/*
  if (my_msg->data[0] == set_state){
    if ((var_ptr->state = my_msg->data[1]) > 0){
      set_relais(my_hw_nr);
    }
    else{
      clear_relais(my_hw_nr);
    }
    // spi_relais(rel1, rel2, rel3);
  }
*/
// if (my_msg->data[0] == get_state){}   // msg mit state-Zustand verschicken


  if ((*my_msg)[can_msg_data] == key_down_released){
   // Taste Down kurz gedrckt: komplett abwrtsfahren
    var_ptr->state = jalousie_all_down;
    var_ptr->sek   = pgm_read_byte(&(my_module->parm1));
    var_ptr->timer = t_second;
    set_relais(my_hw_nr);
    clear_relais(my_hw_nr+1);
    // spi_relais(rel1, rel2, rel3);
  }

  else if ((*my_msg)[can_msg_data] == key_released){
   // Taste Up kurz gedrckt: komplett aufwrtsfahren
    var_ptr->state = jalousie_all_up;
    var_ptr->sek   = pgm_read_byte(&(my_module->parm1));
    var_ptr->timer = t_second;
    set_relais(my_hw_nr);
    set_relais(my_hw_nr+1);
    // spi_relais(rel1, rel2, rel3);
  }

  else if ((*my_msg)[can_msg_data] == key_down_long_pressed){
    var_ptr->state = jalousie_manual_down;
    var_ptr->sek   = 0;
    var_ptr->timer = 0;
    set_relais(my_hw_nr);
    clear_relais(my_hw_nr+1);
    // spi_relais(rel1, rel2, rel3);
  }

  else if ((*my_msg)[can_msg_data] == key_long_pressed){
    var_ptr->state = jalousie_manual_up;
    var_ptr->sek   = 0;
    var_ptr->timer = 0;
    set_relais(my_hw_nr);
    set_relais(my_hw_nr+1);
    // spi_relais(rel1, rel2, rel3);
  }

  else if ( ((*my_msg)[can_msg_data] == key_down_long_released) ||
            ((*my_msg)[can_msg_data] == key_long_released))
  {
    var_ptr->state = 0;
    var_ptr->sek   = 0;
    var_ptr->timer = 0;
    clear_relais(my_hw_nr);
    clear_relais(my_hw_nr+1);
    // spi_relais(rel1, rel2, rel3);
  }
}


// End of file


