
#include <avr/interrupt.h>
#include <avr/signal.h>

#include <avr/io.h>
#include <inttypes.h>
#include <pgmspace.h>

#include <spi.h>
#include <mcp2515.h>
#include <buskoppler.h>


/************************************************************************/
/*
/*
/*
/************************************************************************/

#define set_bit(sfr,bit)	(_SFR_BYTE(sfr) |= _BV(bit))
#define clear_bit(sfr,bit)	(_SFR_BYTE(sfr) &= ~_BV(bit))


void spi_master_init(void){
  spi_ddr  |= _BV(spi_sclk_bit) |
              _BV(spi_mosi_bit) |
              _BV(spi_cs0_bit)  |
              _BV(spi_cs1_bit)  |
              _BV(spi_cs2_bit)  |
              _BV(spi_cs3_bit);

  spi_ddr  &= (uint8_t)
              ~_BV(spi_miso_bit);

 spi_port  |= _BV(spi_cs0_bit)  |
              _BV(spi_cs1_bit)  |
              _BV(spi_cs2_bit)  |
              _BV(spi_cs3_bit);

  spi_port &= (uint8_t)
              ~(_BV(spi_sclk_bit) &
              _BV(spi_mosi_bit));

  SPCR      = 0x50;
  SPSR	    = 0x01;  //0x00;  //0x01;
}


uint8_t spi_byte(uint8_t spi_data){
  SPDR = spi_data;
  while(!SPSR & _BV(SPIF));
  return(SPDR);
}


void spi_block(uint8_t *spi_send, uint8_t *spi_receive, uint8_t anz){
  while(anz--){
    SPDR = *spi_send++;
    while(!(SPSR & _BV(SPIF)));
    *spi_receive++ = SPDR;
  }
}


void spi_relais(uint8_t r1, uint8_t r2, uint8_t r3){
  spi_port &= ~_BV(spi_cs3_bit);
  SPDR = r1;
  while(!(SPSR & _BV(SPIF)));
  SPDR = r2;
  while(!(SPSR & _BV(SPIF)));
  SPDR = r3;
  while(!(SPSR & _BV(SPIF)));
  spi_port |= _BV(spi_cs3_bit);
}


/************************************************************************/
/*
/* spi_mcp2515_0 / spi_mcp2515_1
/*
/* Datenbertragung an den MCP2515 (0/1)
/* anz Bytes werden von Adresse spi_ptr an den MCP2515 bertragen.
/* Gleichzeitig werden vom MCP2515 empfangene Daten nach Adresse spi_ptr gespeichert.
/*
/************************************************************************/

void spi_mcp2515_0(uint8_t *spi_ptr, uint8_t anz){
  spi_port &= ~_BV(spi_cs0_bit);
  while(anz--){
    SPDR = *spi_ptr;
    while(!(SPSR & _BV(SPIF)));
    *spi_ptr++ = SPDR;
  }
  spi_port |= _BV(spi_cs0_bit);
}


void spi_mcp2515_1(uint8_t *spi_ptr, uint8_t anz){
  spi_port &= ~_BV(spi_cs1_bit);
  while(anz--){
    SPDR = *spi_ptr;
    while(!(SPSR & _BV(SPIF)));
    *spi_ptr++ = SPDR;
  }
  spi_port |= _BV(spi_cs1_bit);
}


/************************************************************************/
/*
/* spi_const_mcp2515_0 / spi_const_mcp2515_1
/*
/* Senden konstanter Daten den MCP2515
/* Das erste Byte, auf das konstant_ptr zeigt, gibt die Anzahl der zu sendenden Bytes an.
/* In buf werden Daten des MCP2515 zurckgegeben
/*
/************************************************************************/

void spi_const_mcp2515_0(uint8_t *buf, const uint8_t *constant_ptr){
  uint8_t anz = pgm_read_byte(constant_ptr);
  memcpy_P(buf, constant_ptr+1, anz);
  spi_mcp2515_0(buf, anz);
}


void spi_const_mcp2515_1(uint8_t *buf, const uint8_t *constant_ptr){
  uint8_t anz = pgm_read_byte(constant_ptr);
  memcpy_P(buf, constant_ptr+1, anz);
  spi_mcp2515_1(buf, anz);
}


/************************************************************************/
/*
/*
/*
/************************************************************************/
/*
#define SPI_READ_RXBUF0	0x90
#define SPI_READ_RXBUF1	0x94
#define SPI_LOAD_TXBUF0 0x40
#define SPI_LOAD_TXBUF1 0x42
#define SPI_LOAD_TXBUF2 0x44
#define SPI_RTS0 	0x81
#define SPI_RTS1 	0x82
#define SPI_RTS2	0x84

#define SPI_READ_STATUS 0xA0
*/
/************************************************************************/
/*
/*
/*
/************************************************************************/
/*
void mcp0_rxd_buf0(uint8_t *buf){
  *buf = SPI_READ_RXBUF0;
  spi_mcp2515_0(buf, (*(buf+5))+6);
}


void mcp0_rxd_buf1(uint8_t *buf){
  *buf = SPI_READ_RXBUF1;
  spi_mcp2515_0(buf, (*(buf+5))+6);
}


void mcp1_rxd_buf0(uint8_t *buf){
  *buf = SPI_READ_RXBUF0;
  spi_mcp2515_1(buf, (*(buf+5))+6);
}


void mcp1_rxd_buf1(uint8_t *buf){
  *buf = SPI_READ_RXBUF1;
  spi_mcp2515_1(buf, (*(buf+5))+6);
}
*/

/************************************************************************/
/*
/* <TXD-Befehl> <CAN-ID0-3> <Byte-Anzahl> <Datenbytes 0-7>
/*
/************************************************************************/
/*
void mcp0_txd_buf0(uint8_t *buf){
  *buf = SPI_LOAD_TXBUF0;
  spi_mcp2515_0(buf, (*(buf+5))+6);
  *buf = SPI_RTS0;
  spi_mcp2515_0(buf, 1);
}


void mcp0_txd_buf1(uint8_t *buf){
  *buf = SPI_LOAD_TXBUF1;
  spi_mcp2515_0(buf, (*(buf+5))+6);
  *buf = SPI_RTS1;
  spi_mcp2515_0(buf, 1);
}


void mcp0_txd_buf2(uint8_t *buf){
  *buf = SPI_LOAD_TXBUF2;
  spi_mcp2515_0(buf, (*(buf+5))+6);
  *buf = SPI_RTS2;
  spi_mcp2515_0(buf, 1);
}


void mcp1_txd_buf0(uint8_t *buf){
  *buf = SPI_LOAD_TXBUF0;
  spi_mcp2515_1(buf, (*(buf+5))+6);
  *buf = SPI_RTS0;
  spi_mcp2515_1(buf, 1);
}


void mcp1_txd_buf1(uint8_t *buf){
  *buf = SPI_LOAD_TXBUF1;
  spi_mcp2515_1(buf, (*(buf+5))+6);
  *buf = SPI_RTS1;
  spi_mcp2515_1(buf, 1);
}


void mcp1_txd_buf2(uint8_t *buf){
  *buf = SPI_LOAD_TXBUF2;
  spi_mcp2515_1(buf, (*(buf+5))+6);
  *buf = SPI_RTS2;
  spi_mcp2515_1(buf, 1);
}
*/


